import wget
import pathlib
import zipfile
import os

import numpy as np

import pytest
import pickle

from piscat.InputOutput import StatusLine, DirectoryType, reading_videos, video_reader
from piscat.Visualization import JupyterDisplay_StatusLine, JupyterDisplay
from piscat.Preproccessing import Normalization
from piscat.BackgroundCorrection import DifferentialRollingAverage, NoiseFloor

data_path = pathlib.Path(__file__).parent / 'data'


@pytest.fixture
def control_video_path():

    video_path =  data_path / 'Control' / 'control_4999_128_128_uint16_2.33FPS.raw'
    zip_file = data_path / 'Control.zip'
    if not os.path.exists(video_path):
        zip_file = wget.download('https://owncloud.gwdg.de/index.php/s/tzRZ7ytBd1weNDl/download')
        zip = zipfile.ZipFile(zip_file)
        zip.extractall(data_path)
        zip.close()
    if os.path.exists(zip_file):
        os.remove(zip_file)

    return video_path


@pytest.fixture()
def control_video(control_video_path):
    return reading_videos.video_reader(file_name=control_video_path, type='binary', img_width=128, img_height=128,
                                        image_type=np.dtype('<u2'), s_frame=0, e_frame=-1)


def test_can_read_raw_video(control_video_path):

    df_video = DirectoryType(data_path / 'Control', type_file='raw').return_df()
    video = video_reader(file_name=control_video_path, type='binary', img_width=128, img_height=128,
                                        image_type=np.dtype('<u2'), s_frame=0, e_frame=-1)

    assert len(df_video) == 1
    assert isinstance(video, np.ndarray)
    assert video.shape == (4999, 128, 128)


def test_statusline(control_video):
    assert JupyterDisplay_StatusLine(control_video, median_filter_flag=False, color='gray', imgSizex=5, imgSizey=5,
                              IntSlider_width='500px',
                              step=1)


def test_normalization(control_video):
    video_pn, power_fluctuation = Normalization(video=control_video).power_normalized()

    assert (video_pn >= 0).all()
    assert np.max(video_pn) == pytest.approx(3611, 1)
    assert power_fluctuation.shape == (4999, )
    assert video_pn.shape == (4999, 128, 128)


def test_background_correction_dra(control_video):

    DRA_PN = DifferentialRollingAverage(video=control_video, batchSize=200,)
    RVideo_PN_ = DRA_PN.differential_rolling(FFT_flag=False)

    with open(data_path / 'expected_dra_result_1.pck', 'rb') as f:
        expected = pickle.load(f)

    np.testing.assert_array_almost_equal(RVideo_PN_[0, ...], expected)
    assert len(control_video) > len(RVideo_PN_)


def _test_tutorial2(control_video):
    control_video = control_video[:500, ...]
    status_ = StatusLine(control_video)  # Reading the status line
    video_remove_status, status_information = status_.find_status_line()  # Examining the status line & removing it

    # Normalization of the power in the frames of a video
    video_pn, _ = Normalization(video=video_remove_status).power_normalized()

    DRA_PN = DifferentialRollingAverage(video=video_pn, batchSize=200)
    RVideo_PN_, _ = DRA_PN.differential_rolling(FFT_flag=False)
    disp = JupyterDisplay(RVideo_PN_, median_filter_flag=False, color='gray', imgSizex=5, imgSizey=5, IntSlider_width='500px', step=100)
    l_range = list(range(30, 200, 30))
    noise_floor_DRA_pn = NoiseFloor(video_pn, list_range=l_range, n_jobs=1)



import matplotlib
from PyQt5 import QtCore, QtWidgets
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure


class MplCanvas(FigureCanvas):

    def __init__(self, parent=None, width=5, height=4, dpi=100):
        fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = fig.add_subplot(111)
        super(MplCanvas, self).__init__(fig)


class UpdatingPlots(QtWidgets.QMainWindow):

    def __init__(self, *args, **kwargs):
        super(UpdatingPlots, self).__init__(*args, **kwargs)

        self.canvas = MplCanvas(self, width=6, height=6, dpi=100)
        self.setCentralWidget(self.canvas)

    def update_plot(self, ydata):
        self.canvas.axes.cla()  # Clear the canvas.
        self.canvas.axes.plot(ydata, 'r-')
        self.canvas.axes.grid()
        self.canvas.axes.set_ylabel('Pixel intensity')
        self.canvas.draw()



import os
import time
import cv2
import imageio
from tqdm.autonotebook import tqdm


def write_binary(dir_path, file_name, data, type='original'):
    """
    This function writes video as a binary.

    Parameters
    ----------
    dir_path: str
        Path of the directory that video save on it.

    file_name: str
       Name of the save file.

    data: NDArray
        Video with numpy format.

    type: str or bin_type
        The video bin type is not changed by 'original,' but the user can convert it (e.g. float --> int16).
    """
    timestr = time.strftime("%Y%m%d-%H%M%S")
    try:
        dr_mk = os.path.join(dir_path, timestr)
        os.mkdir(dr_mk)
        print("Directory ", timestr, " Created ")
    except FileExistsError:
        print("Directory ", timestr, " already exists")

    save_path = os.path.join(dir_path, timestr, file_name)

    if type == 'original':
        data = data
    else:
        data = data.astype(type)

    data = data.copy(order='C')
    with open(save_path, 'wb') as outfile:
        outfile.write(data)


def write_MP4(dir_path, file_name, data, fps=10):
    """
    This function writes video as a MP4.

    Parameters
    ----------
    dir_path: str
        Path of the directory that video save on it.

    file_name: str
       Name of the save file.

    data: NDArray
        Video with numpy format.

    fps: int
        Number of frame per seconds.
    """
    timestr = time.strftime("%Y%m%d-%H%M%S")
    try:
        dr_mk = os.path.join(dir_path, timestr)
        os.mkdir(dr_mk)
        print("Directory ", timestr, " Created ")
    except FileExistsError:
        print("Directory ", timestr, " already exists")

    save_path = os.path.join(dir_path, timestr, file_name)
    data = data.copy(order='C')
    out = cv2.VideoWriter(save_path, cv2.VideoWriter_fourcc(*'mp4v'), fps, (data.shape[1], data.shape[2]), False)

    for fr_ in tqdm(range(data.shape[0])):
        frame = data[fr_, ...]
        out.write(frame)
    out.release()


def write_GIF(dir_path, file_name, data, jump=0, fps=10):
    """
    This function writes video as a GIF.

    Parameters
    ----------
    dir_path: str
        Path of the directory that video save on it.

    file_name: str
       Name of the save file

    data: NDArray
        Video with numpy format.

    jump: int
        Define stride between frames

    fps: int
        Number of frame per seconds
    """
    timestr = time.strftime("%Y%m%d-%H%M%S")
    try:
        dr_mk = os.path.join(dir_path, timestr)
        os.mkdir(dr_mk)
        print("Directory ", timestr, " Created ")
    except FileExistsError:
        print("Directory ", timestr, " already exists")

    save_path = os.path.join(dir_path, timestr, file_name)
    data = data.copy(order='C')
    image_ = []
    for frame_number in tqdm(range(0, data.shape[0] - jump, jump)):
        image_.append(data[frame_number, ...])

    imageio.mimsave(save_path, image_, fps=fps)
